#!/bin/bash
###########################################################################
# Setup moodle for the Open School Server
###########################################################################
# (c) 2006 by Lars Rupp
#
INCDIR=/srv/www/inc
ROOTDIR=/srv/www/moodle
DATADIR=/srv/www/moodledata
LINKDIR=/srv/www/oss
LOGFILE=/var/log/moodle-setup.log
VERBOSE="no"

###########################################################################
# Warning: don't change anything below this line
###########################################################################

function LOG() {
    LOG_DATE=`date "+%b %d %H:%M:%S"`
    HOST=`hostname`
    echo "$LOG_DATE $HOST setup: $1" >> $LOGFILE
    if [ "$VERBOSE" = "yes" ]; then
        echo $1
    fi
}

if [ ! -f /etc/sysconfig/schoolserver ]; then
	echo "This is no OSS!"
	exit 1
fi

PATH=$PATH:/sbin:/usr/sbin:/bin
MYROOT="/usr/share/doc/packages/moodle"
CHAT_SERVERHOST="schoolserver"
LDAP_SUFFIX=`grep suffix /etc/openldap/slapd.conf | cut -d "\"" -f 2`
LDAP_DN=`cat /etc/openldap/slapd.conf | grep -v "#" | awk '/rootdn/ {print }' | cut -d "\"" -f 2`
LDAP_SUFFIX=`cat /etc/openldap/slapd.conf | grep -v "#" | awk '/suffix/ {print }' | cut -d "\"" -f 2`
LDAP_CONTEXT="ou=people,$LDAP_SUFFIX"
LANG=`ldapsearch -x uid=admin preferredLanguage | grep preferredLanguage: | awk '" " { print $2 }'`
#SCHOOLNAME=`ldapsearch -x uid=admin o | grep ^o: | awk '" " { print $2 }'`
SCHOOLNAME=`echo "uid admin 
attributes o" | oss_get_user.pl text | grep '^o ' | sed 's/o //'`
# admin settings...
ADMINPASS=`get_admin_pw`
PASS=`mktemp XXXXXX`
SESSIONKEY=`mktemp XXXXXXXX`
ADMIN_LANG=`ldapsearch -x uid=admin preferredLanguage | grep preferredLanguage: | awk '" " { print $2 }'`
admin_lang_lowercase=` echo $(echo $LANG | tr [:upper:] [:lower:])`
admin_lang_lowercase=${admin_lang_lowercase}_utf8
ADMIN_FIRST_NAME=`ldapsearch -x uid=admin givenName | grep givenName: | awk '" " { print $2 }'`
if [ "$ADMIN_FIRST_NAME" == "" ] ; then
	ADMIN_FIRST_NAME="System"
fi
ADMIN_LAST_NAME=`ldapsearch -x uid=admin sn | grep sn: | awk '" " { print $2 }'`
if [ "$ADMIN_LAST_NAME" == "" -o "$ADMIN_LAST_NAME" == "System" ] ; then
	ADMIN_LAST_NAME="Administrator"
fi
ADMIN_CITY=`ldapsearch -x uid=admin l | grep l: | awk '" " { print $2 }'`
if [ "$ADMIN_CITY" == "" ] ; then
	ADMIN_CITY=""
fi

###########################################################################
# checks...
###########################################################################

. /etc/sysconfig/schoolserver

# check if we are using the correct server:
DBHOST=`ifconfig | grep -q $SCHOOL_MAILSERVER; echo $?`
if [ $DBHOST -ne 0 ]; then
	echo "This is not the mailserver!"
	exit 1
fi
DBHOST="mailserver"

###########################################################################
# start installation
###########################################################################
test -d $DATADIR || mkdir -p $DATADIR/{1,2,backup,cache,quarantine,sessions,temp,users}
cd $DATADIR
LOG "`basename $0` started" 
LOG "Setting global permissions of $DATADIR" 
chown -R wwwrun:www $DATADIR 2>>$LOGFILE

# check if we can use ACLs...
DEVICE=`df -h $DATADIR | tail -n 1 | awk '{ print $1 }'`
WITH_ACL=`mount | grep $DEVICE | grep -q acl ; echo $?`

if [ $WITH_ACL -eq 0 ] ; then
	LOG "Setting default ACLs for $DATADIR" >> $LOGFILE
	setfacl -m u:wwwrun:--x /home/software 2>>$LOGFILE
	setfacl -R -m g:teachers:--- $DATADIR/* 2>>$LOGFILE
	setfacl -R -m g:students:--- $DATADIR/* 2>>$LOGFILE
	setfacl -R -m d:g:teachers:r-x $DATADIR/1 2>>$LOGFILE
	setfacl -R -m d:g:students:r-x $DATADIR/2 2>>$LOGFILE
else
	LOG "WARNING: NO ACL SUPPORT FOR $DATADIR DETECTED!" 
fi

cd $MYROOT


###################################
# start mysql database if necessary
###################################
if [ `/etc/init.d/mysql status >/dev/null; echo $?`x != "0"x ]; then
    /sbin/insserv mysql
    /etc/init.d/mysql start
fi

###################################
# the plugins 
###################################
test -d /usr/share/oss/plugins/add_user || mkdir -p /usr/share/oss/plugins/add_user
test -d /usr/share/oss/plugins/del_user || mkdir -p /usr/share/oss/plugins/del_user
cp -b $MYROOT/moodle-add-user /usr/share/oss/plugins/add_user/
cp -b $MYROOT/moodle-del-user /usr/share/oss/plugins/del_user/
chown root:root /usr/share/oss/plugins/del_user/moodle-del-user
chown root:root /usr/share/oss/plugins/add_user/moodle-add-user
chmod 750 /usr/share/oss/plugins/del_user/moodle-del-user
chmod 750 /usr/share/oss/plugins/add_user/moodle-add-user

###################################
# the moodle-config.php file
###################################
LOG "Creating new $INCDIR/moodle-config.php"

if [ -f $INCDIR/moodle-config.php.backup ]; then
	LOG "ERROR: $INCDIR/moodle-config.php.backup exists!" 
	LOG "Leaving old $INCDIR/moodle-config.php.backup"
	LOG "Deleting $INCDIR/moodle-config.php file instead"
	rm $INCDIR/moodle-config.php 2>/dev/null
	cp $INCDIR/moodle-config.php.backup $INCDIR/moodle-config.php
else
	cp $INCDIR/moodle-config.php $INCDIR/moodle-config.php.backup
fi
chown wwwrun:root $INCDIR/moodle-config.php 2>>$LOGFILE
chmod 460 $INCDIR/moodle-config.php 2>>$LOGFILE

# adapt template to the running system
INTERNURL="https://schoolserver.$SCHOOL_DOMAIN/moodle"
perl -pi -e "s#DBHOST#$DBHOST#g" $INCDIR/moodle-config.php  2>>$LOGFILE
perl -pi -e "s#PASSWD#$PASS#g" $INCDIR/moodle-config.php 2>>$LOGFILE
perl -pi -e "s#WWWROOT#$INTERNURL#g" $INCDIR/moodle-config.php 2>>$LOGFILE
perl -pi -e "s#ROOTDIR#$ROOTDIR#g" $INCDIR/moodle-config.php 2>>$LOGFILE
perl -pi -e "s#DATADIR#$DATADIR#g" $INCDIR/moodle-config.php 2>>$LOGFILE
perl -pi -e "s#SCHOOL_DOMAIN#$SCHOOL_DOMAIN#g" $INCDIR/moodle-config.php 2>>$LOGFILE
perl -pi -e "s#http://localhost/moodle#$INTERNURL#g" $INCDIR/moodle-config.php 2>>$LOGFILE

# try to prepare for use of dyndns (moodle via masquerading)
DYNDNSURL=`grep -v \# /etc/ddclient.conf | grep -v \\ | perl -pe 's/^\s+$//g'`
if [ "$DYNDNSURL" == "" ]; then
  EXTERNURL="http://schulserver.$SCHOOL_DOMAIN/moodle"
else
  EXTERNURL="https://$DYNDNSURL/moodle"
fi
SUBNET=`echo ${SCHOOL_MAILSERVER%\.*\.*}`
perl -pi -e "s#INTERNURL#$INTERNURL#g" $INCDIR/moodle-config.php 2>>$LOGFILE
perl -pi -e "s#EXTERNURL#$EXTERNURL#g" $INCDIR/moodle-config.php 2>>$LOGFILE
perl -pi -e "s#SUBNET#$SUBNET#g" $INCDIR/moodle-config.php 2>>$LOGFILE

###################################
# adapt apache config
###################################
#
# /etc/apache2/mod_log_config.conf
MOD_LOG_FILE="/etc/apache2/mod_log_config.conf"
if [ `grep -q moodleformat $MOD_LOG_FILE; echo $?` -eq 1 ] ; then
LOG "Adding Logfile format to $MOD_LOG_FILE"

cat >> $MOD_LOG_FILE <<-EOF
	# To allow a better report status of moodle actions the following line is added for moodleformat
	LogFormat "%h %l %{MOODLEUSER}n %t \"%r\" %s %b \"%{Referer}i\" \"%{User-Agent}i\"" moodleformat
EOF

else 
	LOG "WARNING: Logformat for moodle exists in $MOD_LOG_FILE - skipping"
fi

# /etc/apache2/vhosts.d/oss/moodle_include.conf
if [ ! -f /etc/apache2/vhosts.d/oss/moodle_include.conf ]; then
	LOG "Preparing apache2 directory"
    cp /etc/apache2/conf.d/moodle_include.conf /etc/apache2/vhosts.d/oss/ 2>>$LOGFILE
    mv /etc/apache2/conf.d/moodle_include.conf /etc/apache2/vhosts.d/oss-ssl/ 2>>$LOGFILE
else
	LOG "WARNING: moodle_include.conf is already included in /etc/apache2/vhosts.d/oss*/" 
fi

perl -pi -e "s#ROOTDIR#$ROOTDIR#" /etc/apache2/vhosts.d/oss-ssl/moodle_include.conf 2>>$LOGFILE
perl -pi -e "s#ROOTDIR#$ROOTDIR#" /etc/apache2/vhosts.d/oss/moodle_include.conf 2>>$LOGFILE


###################################
# create the database
###################################

if [ `echo "show databases;" | mysql -u root | grep -q moodle; echo $?` -ne 0 ] ; then 
	LOG "Creating Database" 
	mysqladmin -u root CREATE moodle --default-character-set=utf8;
	echo "GRANT SELECT,INSERT,UPDATE,DELETE,CREATE,CREATE TEMPORARY TABLES,DROP,INDEX,ALTER ON moodle.* TO moodleuser@localhost IDENTIFIED BY '$PASS';" \
     | mysql -u root  2>>$LOGFILE
	echo "GRANT SELECT,INSERT,UPDATE,DELETE,CREATE,CREATE TEMPORARY TABLES,DROP,INDEX,ALTER ON moodle.* TO moodleuser@$SCHOOL_DOMAIN IDENTIFIED BY '$PASS';" \
     | mysql -u root 2>>$LOGFILE
	mysqladmin -u root reload;

	MD5PASSWORD=`/usr/bin/php $MYROOT/create-password.php $ADMINPASS`

	# adapt moodle-inserts.sql to local settings
	cp $MYROOT/moodle-database.sql $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	perl -pi -e "s#DOMAIN#$SCHOOL_DOMAIN#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	perl -pi -e "s#LDAP_CONTEXT#$LDAP_CONTEXT#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	perl -pi -e "s#LDAP_SUFFIX#$LDAP_SUFFIX#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	perl -pi -e "s#DATADIR#$DATADIR#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	perl -pi -e "s#ADMIN_LANG#$ADMIN_LANG#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	perl -pi -e "s#admin_lang_lowercase#$admin_lang_lowercase#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	perl -pi -e "s#SCHOOL_MAILSERVER#$SCHOOL_MAILSERVER#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	perl -pi -e "s#MD5PASSWORD#$MD5PASSWORD#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	perl -pi -e "s#SCHOOLNAME#$SCHOOLNAME#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE

	perl -pi -e "s#ADMIN_FIRST_NAME#$ADMIN_FIRST_NAME#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	perl -pi -e "s#ADMIN_LAST_NAME#$ADMIN_LAST_NAME#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	perl -pi -e "s#ADMIN_CITY#$ADMIN_CITY#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	perl -pi -e "s#CHAT_SERVERHOST#$CHAT_SERVERHOST#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
    perl -pi -e "s#SESSIONKEY#$SESSIONKEY#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
#	perl -pi -e "s#LDAPBASE#$LDAP_DN#" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	perl -pi -e "s#QUARANTINEDIR#$DATADIR/quarantine#g" $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	

	# dump the correct inserts in the database
	mysql -u root moodle < $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
	rm $MYROOT/moodle-inserts.sql.in 2>>$LOGFILE
else
	LOG "WARNING: Database \"moodle\" exists - skipping"
fi

###################################
# prepare directory for OSS homepage 
###################################
if [ ! -L $LINKDIR/moodle ]; then
    LOG "Creating Symlink to $LINKDIR/moodle"
    pushd $LINKDIR 1>/dev/null 2>>$LOGFILE
	ln -s $ROOTDIR moodle
    popd 1>/dev/null 2>>$LOGFILE
fi
LOG "Calling $LINKDIR/index.sh"
/bin/bash $LINKDIR/./index.sh >  $LINKDIR/./index.html

###########################################################################
# finished
###########################################################################
rm -f $PASS
LOG "setup-moodle finished"
rcapache2 restart 1>/dev/null 2>>$LOGFILE
