#!/bin/bash
###########################################################################
# Uninstall moodle for the Open School Server
###########################################################################
# (c) 2006 by Lars Rupp
#
INCDIR=/etc
ROOTDIR=/srv/www/htdocs/moodle
DATADIR=/srv/www/moodledata
LINKDIR=/srv/www/oss
LOGFILE=/var/log/setup-moodle.log
VERBOSE="no"

###########################################################################
# Warning: don't change anything below this line
###########################################################################
function LOG() {
    LOG_DATE=`date "+%b %d %H:%M:%S"`
    HOST=`hostname`
    echo "$LOG_DATE $HOST uninstall: $1" >> $LOGFILE
    if [ "$VERBOSE" = "yes" ]; then
        echo $1
    fi
}

if [ ! -f /etc/sysconfig/schoolserver ]; then
	echo "This is no OSS!"
	exit 1
fi

PATH=$PATH:/sbin:/usr/sbin:/bin
MYROOT=`pwd`
ADMINASS=`get_admin_pw`

###########################################################################
# checks...
###########################################################################

. /etc/sysconfig/schoolserver

# check if we are using the correct server:
DBHOST=`ifconfig | grep -q $SCHOOL_MAILSERVER; echo $?`
if [ $DBHOST -ne 0 ]; then
	echo "This is not the mailserver!"
	exit 1
fi
DBHOST="mailserver"

###########################################################################
# start uninstall 
###########################################################################
LOG "started"
###################################
# remove the datadir
###################################
LOG "removing $DATADIR"
rm -r $DATADIR 2>>$LOGFILE
###################################
# remove the linkdir
###################################
LOG "removing $LINKDIR/moodle"
rm -r $LINKDIR/moodle 2>>/LOGFILE
###################################
# remove the database
###################################
if [ ! `echo "show databases;" | mysql -u root | grep -q moodle; echo $?` -ne 0 ] ; then 
	LOG "Removing Database" 
	mysqladmin -u root -f DROP moodle 1>/dev/null 2>>$LOGFILE
else
	LOG "WARNING: Can't find Database \"moodle\""           
fi

###################################
# remove apache2 entries
###################################
LOG "Removing apache include file" 
find /etc/apache2/vhosts.d/ -name "moodle_include.conf" -exec rm -v {} \; >> $LOGFILE 2>&1

MOD_LOG_FILE="/etc/apache2/mod_log_config.conf"
LOG "Removing apache2 logfile config"
if [ `grep -q moodleformat $MOD_LOG_FILE; echo $?` -eq 0 ] ; then
	perl -pi -e "s/(.*)moodleformat\n$//g" $MOD_LOG_FILE
else
    LOG "WARNING: Logformat for moodle doesn't exist in $MOD_LOG_FILE - skipping" 
fi

###################################
# remove config files
###################################
LOG "Removing /etc/moodle_config.php"
rm /etc/moodle-config.php 2>> $LOGFILE

###########################################################################
# finished
###########################################################################
LOG "uninstall-moodle finished"
rcapache2 restart 1>/dev/null 2>>$LOGFILE

