#!/bin/bash
###########################################################################
# Update moodle for the Open School Server
###########################################################################
# (c) 2006 by Lars Rupp
#
INCDIR=/srv/www/inc
ROOTDIR=/srv/www/moodle
DATADIR=/srv/www/moodledata
LINKDIR=/srv/www/oss
LOGFILE=/var/log/moodle-setup.log

###########################################################################
# Warning: don't change anything below this line
###########################################################################
function LOG() {
    LOG_DATE=`date "+%b %d %H:%M:%S"`
    HOST=`hostname`
    echo "$LOG_DATE $HOST update: $1" >> $LOGFILE
    if [ "$VERBOSE" = "yes" ]; then
        echo $1
    fi
}

if [ ! -f /etc/sysconfig/schoolserver ]; then
	echo "This is no OSS!"
	exit 1
fi

PATH=$PATH:/sbin:/usr/sbin:/bin
MYROOT=`pwd`
CHAT_SERVERHOST="schoolserver"
LDAP_SUFFIX=`grep suffix /etc/openldap/slapd.conf | cut -d "\"" -f 2`
LDAP_DN=`cat /etc/openldap/slapd.conf | grep -v "#" | awk '/rootdn/ {print }' | cut -d "\"" -f 2`
LDAP_SUFFIX=`cat /etc/openldap/slapd.conf | grep -v "#" | awk '/suffix/ {print }' | cut -d "\"" -f 2`
LDAP_CONTEXT="ou=people,$LDAP_SUFFIX"
LANG=`ldapsearch -x uid=admin preferredLanguage | grep preferredLanguage: | awk '" " { print $2 }'`
# admin settings...
PASS=`get_admin_pw`
ADMIN_LANG=`ldapsearch -x uid=admin preferredLanguage | grep preferredLanguage: | awk '" " { print $2 }'`
admin_lang_lowercase=` echo $(echo $LANG | tr [:upper:] [:lower:])`
ADMIN_FIRST_NAME=`ldapsearch -x uid=admin givenName | grep givenName: | awk '" " { print $2 }'`
if [ "$ADMIN_FIRST_NAME" == "" ] ; then
	ADMIN_FIRST_NAME="System"
fi
ADMIN_LAST_NAME=`ldapsearch -x uid=admin sn | grep sn: | awk '" " { print $2 }'`
if [ "$ADMIN_LAST_NAME" == "" ] ; then
	ADMIN_LAST_NAME="Administrator"
fi
ADMIN_CITY=`ldapsearch -x uid=admin l | grep l: | awk '" " { print $2 }'`
if [ "$ADMIN_CITY" == "" ] ; then
	ADMIN_CITY=""
fi

###########################################################################
# checks...
###########################################################################

. /etc/sysconfig/schoolserver

# check if we are using the correct server:
DBHOST=`ifconfig | grep -q $SCHOOL_MAILSERVER; echo $?`
if [ $DBHOST -ne 0 ]; then
	echo "This is not the mailserver!"
	exit 1
fi
DBHOST="mailserver"

###########################################################################
# start installation
###########################################################################
cd $DATADIR
LOG "started"
LOG "Setting global permissions of $DATADIR" 
chown -R wwwrun $DATADIR 2 >> $LOGFILE

# check if we can use ACLs...
DEVICE=`df -h $DATADIR | tail -n 1 | awk '{ print $1 }'`
WITH_ACL=`mount | grep $DEVICE | grep -q acl ; echo $?`

if [ $WITH_ACL -eq 0 ] ; then
	echo "---- Setting default ACLs for $DATADIR" >> $LOGFILE
	setfacl -m u:wwwrun:--x /home/software 2>>$LOGFILE
	setfacl -R -m g:teachers:--- $DATADIR/* 2>>$LOGFILE
	setfacl -R -m g:students:--- $DATADIR/* 2>>$LOGFILE
	setfacl -R -m d:g:teachers:r-x $DATADIR/1 2>>$LOGFILE
	setfacl -R -m d:g:students:r-x $DATADIR/2 2>>$LOGFILE
else
	LOG "WARNING: NO ACL SUPPORT FOR $DATADIR DETECTED!"
fi

cd $MYROOT

###################################
# start mysql database if necessary
###################################
if [ `pidof mysqld`x = ""x ]; then
    /sbin/insserv mysql
    /etc/init.d/mysql start
fi

###################################
# the plugins
###################################

test -d /usr/share/oss/plugins/add_user || mkdir -p /usr/share/oss/plugins/add_user
test -d /usr/share/oss/plugins/del_user || mkdir -p /usr/share/oss/plugins/del_user
test -f /usr/share/oss/plugins/add_user/moodle-add-user && rm /usr/share/oss/plugins/add_user/moodle-add-user
test -f /usr/share/oss/plugins/del_user/moodle-del-user && rm /usr/share/oss/plugins/del_user/moodle-del-user
#cp -b $MYROOT/moodle-add-user /usr/share/oss/plugins/add_user/
#cp -b $MYROOT/moodle-del-user /usr/share/oss/plugins/del_user/
#chown root:root /usr/share/oss/plugins/del_user/moodle-del-user
#chown root:root /usr/share/oss/plugins/add_user/moodle-add-user
#chmod 750 /usr/share/oss/plugins/del_user/moodle-del-user
#chmod 750 /usr/share/oss/plugins/add_user/moodle-add-user

###################################
# update the database
###################################

if [ ! `echo "show databases;" | mysql -u root | grep -q moodle; echo $?` -ne 0 ] ; then 
	LOG "Updating Database" 
else
	LOG "WARNING: Can't find Database \"moodle\""
fi

###################################
# prepare directory for OSS homepage
###################################
if [ ! -L $LINKDIR/moodle ]; then
	LOG "Creating Symlink to $LINKDIR/moodle"
    pushd $LINKDIR 1>/dev/null 2>>$LOGFILE
    ln -s $ROOTDIR moodle
    popd 1>/dev/null 2>>$LOGFILE
fi

###########################################################################
# finished
###########################################################################
LOG "finished" 
rcapache2 restart 1>/dev/null 2>>$LOGFILE

